@@ -11,10 +11,11 @@ import sys
 import argparse
 from pyThorlabsPM100x.driver import ThorlabsPM100x
 from pyThorlabsPM100x.plots import PlotObject
+from pyThorlabsPM100x.abstract_interface import abstract_interface
 
 graphics_dir = os.path.join(os.path.dirname(__file__), 'graphics')
 
-class interface():
+class interface(abstract_interface):
     """
     Create a high-level interface with the device, and provide a GUI
 
@@ -99,57 +100,17 @@ class interface():
 
     output = {'Power':0}  #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
 
-    def __init__(self, app, mainwindow, name_logger=__package__):
-        # app           = The pyqt5 QApplication() object
-        # mainwindow    = Main Window of the application
-
-        # name_logger   = The name of the logger used for this particular istance of the interface object. If none is specified, the name of package is used as logger name
-
-        self.mainwindow = mainwindow
-        self.app = app
-        self.mainwindow.child = self #make sure that the window embedding this interface knows about its child (this is mainly used to link the closing events when using multiple windows)
- 
+    def __init__(self, **kwargs):
         self.output = {'Power':0} 
         self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
         self.refresh_time = 0.2 #default refresh rate, in seconds
         self.stored_data = [] # List used to store data acquired by device
         self.current_power_string = " " # Last power read from powermeter, as a string
         self.connected_device_name = ''
-        self._verbose = True #Keep track of whether this instance of the interface should produce logs or not
-        self._name_logger = ''
 
         self.instrument = ThorlabsPM100x() 
-        self.name_logger = name_logger #Setting this property will also create the logger,set defaulat output style, and store the logger object in self.logger
-
-    @property
-    def verbose(self):
-        return self.verbose
-
-    @verbose.setter
-    def verbose(self,verbose):
-        #When the verbose property of this interface is changed, we also update accordingly the level of the logger object
-        if verbose: loglevel = logging.INFO
-        else: loglevel = logging.CRITICAL
-        self.logger.setLevel(level=loglevel)
-
-    @property
-    def name_logger(self):
-        return self._name_logger
-
-    @name_logger.setter
-    def name_logger(self,name):
-        #Create logger, and set default output style.
-        self._name_logger = name
-        self.logger = logging.getLogger(self._name_logger)
-        self.verbose = self._verbose #This will automatically set the logger verbosity too
-        if not self.logger.handlers:
-            formatter = logging.Formatter(f"[{self.name_logger}]: %(message)s")
-            ch = logging.StreamHandler()
-            ch.setFormatter(formatter)
-            self.logger.addHandler(ch)
-        self.logger.propagate = False
-        
-        
+        super().__init__(**kwargs)
+ 
     def create_gui(self,parent,plot=False): 
         '''
             parent, QWidget        
@@ -158,7 +119,7 @@ class interface():
             plot, boolean
                 If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
         '''
-        self.parent = parent
+        super().create_gui(parent)
         self.gui = gui(self,plot)
         self.gui.initialize()
         self.parent.setLayout(self.gui.container) 
@@ -180,7 +141,6 @@ class interface():
         #Then we read the list of devices
         self.logger.info(f"Looking for devices...") 
         list_valid_devices = self.instrument.list_devices()
-        print(list_valid_devices )
         if(len(list_valid_devices)>0):
             list_IDNs_and_devices = [dev[1] + " --> " + dev[0] for dev in list_valid_devices] 
             #list_IDNs_and_devices = [i for i in list_IDN] 
@@ -381,42 +341,17 @@ class interface():
         if(self.continuous_read == True):
             (currentPower,power_units) = self.instrument.power
             self.output['Power'] = currentPower
-            #self.output['PowerUnits'] = power_units
-            if hasattr(self,'trigger'):
-                if not(self.trigger==None):
-                    self.logger.info(f"Trigger Sent.")
-                    QtCore.QTimer.singleShot(int(self.trigger[1]), self.trigger[0])    
-
             self.stored_data.append(currentPower)
+            #self.output['PowerUnits'] = power_units
 
-            #StoredPowers_np = np.array(self.StoredPowers)
-            #Fluctuation = 100 * StoredPowers_np[-10:].ptp(axis=0)  / StoredPowers_np[-10:].mean(axis=0)  
-            
-            #self.PowerFluctuationString.set("{:.2f}".format(Fluctuation) + " %")
+            super().update()    
 
-            #super().Update()
             self.current_power_string = f"{currentPower:.2e}" + ' ' +  power_units
             self.gui.edit_Power.setText(self.current_power_string)
             QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.update)
            
         return
 
-    def set_trigger(self,external_function,delay=0):
-        '''
-        This method allows to use this device as a trigger for other operations. Every time that this interface object acquires data from the device (i.e. every time 
-        the function self.update is executed with self.continuous_read == True), the function external_function is also called. external_function must be a valid function which does not 
-        require any input parameter.
-        The optional parameter delay sets a delay (in ms) between the call to the function update and the call to the function external_function
-        '''
-        if external_function == None:
-            self.trigger = None
-            return
-        if not(callable(external_function)):
-            self.logger.error(f"Input parameter external_function must be a valid function")  
-            return  
-        self.logger.info(f"Creating a trigger for this device...")
-        self.trigger = [external_function, delay]
-
     def close(self):
         if (self.instrument.connected == True):
             self.disconnect_device()
@@ -666,7 +601,7 @@ def main():
     
     app = Qt.QApplication(sys.argv)
     window = MainWindow()
-    Interface = interface(app,window) #In this case window is both the MainWindow and the parent of the gui
+    Interface = interface(app=app,mainwindow=window) #In this case window is both the MainWindow and the parent of the gui
     Interface.verbose = not(args.decrease_verbose)
     app.aboutToQuit.connect(Interface.close) 
     Interface.create_gui(window,plot=True)
